import React, { useCallback, useEffect, useState } from 'react';
import { Provider, Preview, Error } from 'jarle';
import MonacoEditor from '@monaco-editor/react';
import { styled, css } from '@storybook/theming';
import { FaAngleRight } from '@react-icons/all-files/fa/FaAngleRight';
import { FaCopy } from '@react-icons/all-files/fa/FaCopy';
import { FaCheck } from '@react-icons/all-files/fa/FaCheck';
import copy from 'copy-to-clipboard';
const Wrapper = styled.div({
    width: '100%',
    overflow: 'hidden',
    borderRadius: '8px',
    border: '1px solid #eaeaea',
});
const LiveContent = styled.div({
    height: '100%',
    padding: '16px',
    overflow: 'auto',
});
const summaryBarCss = css `
  color: #666;
  font-size: 14px;
  height: 42px;
  background: #f5f5f5;
  border-bottom: 1px solid #eaeaea;
  display: flex;
  align-items: center;
  justify-content: space-between;
  padding: 0 16px;
  > div {
    display: flex;
    align-items: center;
  }
  .icon-btn {
    padding: 4px;
    cursor: pointer;
    margin-right: 8px;
    transition: all 0.2s ease;
    &:hover {
      color: #333;
    }
  }
  &.open {
    .icon-btn.open-btn {
      transform: rotate(90deg);
    }
  }

  .copy-result-text {
    color: limegreen;
    margin-right: 8px;
    font-size: 12px;
  }
`;
const SummaryBar = styled.div `
  ${summaryBarCss}
`;
const editorContentCss = css `
  max-height: 600px;
  overflow: hidden;
`;
const EditorContent = styled.div `
  ${editorContentCss}
`;
const Playground = ({ code, scope, providerProps, editorProps, }) => {
    const editorRef = React.useRef();
    const [_code, setCode] = useState(code);
    const [open, setOpen] = useState(false);
    const [copied, setCopied] = useState(false);
    const [editorContentHeight, setEditorContentHeight] = useState(0);
    const onCopy = useCallback((e) => {
        e.stopPropagation();
        e.preventDefault();
        copy(_code);
        setCopied(true);
    }, []);
    const onCodeChange = useCallback((newCode) => {
        setCode(newCode);
    }, []);
    const onMount = useCallback((editor) => {
        editorRef.current = editor;
        editor.onDidContentSizeChange((e) => {
            setEditorContentHeight(e.contentHeight);
            editor.layout();
        });
    }, []);
    useEffect(() => {
        if (copied) {
            setTimeout(() => {
                setCopied(false);
            }, 1000);
        }
    }, [copied]);
    useEffect(() => {
        var _a;
        if (open) {
            if (editorContentHeight === 0) {
                setEditorContentHeight((_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.getContentHeight());
            }
        }
        else {
            setEditorContentHeight(0);
        }
    }, [open, editorContentHeight]);
    return (React.createElement(Wrapper, null,
        React.createElement(LiveContent, null,
            React.createElement(Provider, Object.assign({ scope: scope }, providerProps, { code: _code }),
                React.createElement(Preview, null),
                React.createElement(Error, null))),
        React.createElement(SummaryBar, { className: open ? 'open' : '' },
            React.createElement("div", null,
                React.createElement(FaAngleRight, { className: "icon-btn open-btn", onClick: () => setOpen((state) => !state) }),
                React.createElement("span", null, "Editor")),
            React.createElement("div", null,
                copied && (React.createElement("span", { className: "copy-result-text" },
                    "Copied",
                    React.createElement(FaCheck, null))),
                React.createElement(FaCopy, { className: "icon-btn", title: "copy", onClick: onCopy }))),
        React.createElement(EditorContent, { className: open ? 'open' : '', style: { height: editorContentHeight } },
            React.createElement(MonacoEditor, Object.assign({ width: "100%", language: "javascript", theme: "vs-dark", options: {
                    fontSize: 16,
                    scrollBeyondLastLine: false,
                    minimap: {
                        enabled: false,
                    },
                } }, editorProps, { value: _code, onChange: onCodeChange, onMount: onMount })))));
};
export default Playground;
